@@ -94,10 +94,16 @@ void cachingQueue::run()
                 emit haveCommand(item.command,item.param,item.receiver);
                 it=queue.erase(it);
                 //queue.remove(prio,it.value()); // Will remove ALL matching commands which breaks some things (memory bulk write)
+
+                // If this is a recurring command, add it back into the queue.
                 if (item.recurring && prio != priorityImmediate) {
                     queue.insert(prio,item);
                 }
-                updateCache(false,item.command,item.param,item.receiver);
+
+                // Immediate will be updated by the add command, any other commands should update the cache
+                if (prio != priorityImmediate) {
+                    updateCache(false,item.command,item.param,item.receiver);
+                }
             }
 #ifdef Q_OS_MACOS
             while (!items.isEmpty()) {
@@ -133,43 +139,16 @@ void cachingQueue::interval(qint64 val)
         mutex.unlock();
         waiting.wakeAll();
     }
+    emit intervalUpdated(val);
     qInfo() << "Changing queue interval to" << val << "ms";
 }
 
 funcs cachingQueue::checkCommandAvailable(funcs cmd,bool set)
 {
-
-    // We need to ignore special commands:
-
-    if (cmd == funcSelectVFO)
-        return cmd;
-
-
-    // If we don't have rigCaps yet, simply return the command.
-    if (rigCaps != Q_NULLPTR && cmd != funcNone && !rigCaps->commands.contains(cmd)) {
-        // We don't have the requested command, so lets see if we can change it to something we do have.
-        // WFVIEW functions should use funcMain/Sub commands by default,
-        if (cmd == funcFreq && rigCaps->commands.contains(funcSelectedFreq))
-            cmd = funcSelectedFreq;
-        else if (cmd == funcMode && rigCaps->commands.contains(funcSelectedMode))
-            cmd = funcSelectedMode;
-        // These are fallback commands for older radios that don't have command 25/26
-        else if(cmd == funcMode)
-        {
-            if (set)
-                cmd = funcModeSet;
-            else
-                cmd = funcModeGet;
-        }
-        else if(cmd == funcFreq)
-        {
-            if (set)
-                cmd = funcFreqSet;
-            else
-                cmd = funcFreqGet;
-        }
-        else
-            cmd = funcNone;
+    Q_UNUSED(set)
+    if (rigCaps != Q_NULLPTR && cmd != funcNone && cmd != funcSelectVFO && !rigCaps->commands.contains(cmd)) {
+        // We don't have the requested command!
+        return funcNone;
     }
     return cmd;
 }
@@ -177,69 +156,109 @@ funcs cachingQueue::checkCommandAvailable(funcs cmd,bool set)
 void cachingQueue::add(queuePriority prio ,funcs func, bool recurring, uchar receiver)
 {
     queueItem q(func,recurring,receiver);
-    add(prio,q);
+    add(prio,q,false);
 }
 
-void cachingQueue::add(queuePriority prio ,queueItem item)
-{
-    item.command=checkCommandAvailable(item.command,item.param.isValid());
-    if (item.command != funcNone)
-    {
-        QMutexLocker locker(&mutex);
-        if (!item.recurring || isRecurring(item.command,item.receiver) != prio)
-        {
-            if (item.recurring && prio == queuePriority::priorityImmediate) {
-                qWarning() << "Warning, cannot add recurring command with immediate priority!" << funcString[item.command];
-            } else {
-                if (item.recurring) {
-                    // also insert an immediate command to get the current value "now" (removes the need to get rigstate)                    
-                    queueItem it=item;
-                    it.recurring=false;
-                    it.param.clear();
-                    //qDebug() << "adding" << funcString[item.command] << "recurring" << it.recurring << "priority" << prio << "receiver" << it.receiver;
-                    queue.insert(queue.cend(),priorityImmediate, it);
-                }
-                queue.insert(prio, item);
-            }
-        }
-    }
-}
-
-
 void cachingQueue::addUnique(queuePriority prio ,funcs func, bool recurring, uchar receiver)
 {
     queueItem q(func,recurring, receiver);
-    addUnique(prio,q);
+    add(prio,q,true);
 }
 
-
-
-void cachingQueue::addUnique(queuePriority prio ,queueItem item)
+void cachingQueue::addUnique(queuePriority prio, queueItem item)
 {
-    item.command=checkCommandAvailable(item.command,item.param.isValid());
-    if (item.command != funcNone)
-    {
-        QMutexLocker locker(&mutex);
-        if (item.recurring && prio == queuePriority::priorityImmediate) {
-            qWarning() << "Warning, cannot add unique recurring command with immediate priority!" << funcString[item.command];
-        } else {
-            int count=queue.remove(prio,item);
-            if (count>0)
-                qDebug() << "cachingQueue()::addUnique deleted" << count << "entries from queue for" << funcString[item.command] << "on receiver" << item.receiver;
+    add(prio,item,true);
+}
 
-            if (item.recurring) {
-                // also insert an immediate command to get the current value "now" (removes the need to get initial rigstate)
-                queueItem it = item;
-                it.recurring=false;
-                it.param.clear();
-                queue.insert(queue.cend(),priorityImmediate, it);
-                qDebug() << "adding unique" << funcString[item.command] << "recurring" << item.recurring << "priority" << prio << "receiver" << item.receiver;
+void cachingQueue::add(queuePriority prio ,queueItem item, bool unique)
+{
+    // If the queue isn't running, no point adding to it!
+    if (this->queueInterval != -1)
+    {
+        item.command=checkCommandAvailable(item.command,item.param.isValid());
+        if ((item.receiver != 0xff || rigState.receiver == item.receiver) && item.command != funcNone)
+        {
+            QMutexLocker locker(&mutex);
+
+            // Do not add a duplicate recurring command of the same priority
+            if (!item.recurring || isRecurring(item.command,item.receiver) != prio)
+            {
+                if (item.recurring && prio == queuePriority::priorityImmediate) {
+                    qWarning() << "cachingQueue::add() Warning, cannot add recurring command with immediate priority!" << funcString[item.command];
+                } else {
+                    if (unique) {
+                        int count=queue.remove(prio,item);
+                        if (count>0)
+                            qDebug() << "cachingQueue::add() deleted" << count << "entries from queue for" << funcString[item.command] << "on receiver" << item.receiver;
+                    }
+
+                    // Don't immediately request funcTransceiverId, wait for the queue to run
+                    if (item.recurring && item.command != funcTransceiverId) {
+                        // also insert an immediate command to get the current value "now" (removes the need to get rigstate)
+                        queueItem it=item;
+                        it.recurring=false;
+                        it.param.clear();
+                        queue.insert(queue.cend(),priorityImmediate, it);
+                    }
+                    queue.insert(prio, item);
+                }
+            }
+            //Immediately update the cache (even if we aren't sending a value)
+            if (!item.recurring && prio == priorityImmediate)
+            {
+                updateCache(false,item.command,item.param,item.receiver);
             }
-            queue.insert(prio, item);
         }
     }
 }
 
+vfoCommandType cachingQueue::getVfoCommand(vfo_t vfo,uchar rx, bool set)
+{
+    vfoCommandType cmd;
+    cmd.receiver = rx;
+    cmd.vfo = vfo;
+    if (rigCaps != Q_NULLPTR) {
+        QMutexLocker locker(&mutex);
+
+        if (set) {
+            cmd.modeFunc = ((rigCaps->commands.contains(funcMode)) ? funcMode: funcModeSet) ;
+            cmd.freqFunc = ((rigCaps->commands.contains(funcFreq)) ? funcFreq: funcFreqSet) ;
+        } else {
+            cmd.modeFunc = ((rigCaps->commands.contains(funcMode)) ? funcMode: funcModeGet) ;
+            cmd.freqFunc = ((rigCaps->commands.contains(funcFreq)) ? funcFreq: funcFreqGet) ;
+        }
+
+        if (!rigCaps->hasCommand29) {
+            // This radio has no direct access to each receiver (main/sub
+            if (rigState.vfoMode == vfoModeType_t::vfoModeVfo && (rigState.receiver == 0 && rx == 0))
+            {
+                // This is acting on main
+                if (vfo == vfoA) {
+                    cmd.modeFunc = ((rigCaps->commands.contains(funcSelectedMode)) ? funcSelectedMode: cmd.modeFunc);
+                    cmd.freqFunc = ((rigCaps->commands.contains(funcSelectedFreq)) ? funcSelectedFreq: cmd.freqFunc);
+                } else if (vfo == vfoB){
+                    cmd.modeFunc = ((rigCaps->commands.contains(funcUnselectedMode)) ? funcUnselectedMode: cmd.modeFunc);
+                    cmd.freqFunc = ((rigCaps->commands.contains(funcUnselectedFreq)) ? funcUnselectedFreq: cmd.freqFunc);
+                }
+            }
+            else if (rx == rigState.receiver)
+            {
+                // Requesting receiver is current
+                cmd.receiver = 0;
+            }
+            else
+            {
+                // Requesting receiver is not current.
+                cmd.modeFunc = funcNone;
+                cmd.freqFunc = funcNone;
+                cmd.receiver = 0xff;
+            }
+        }
+    }
+    //qDebug(logRig()) << "Queue VFO:" << vfo << "rx:" << rx<< "set:" << set << "mode:" << funcString[cmd.modeFunc] << "freq:" << funcString[cmd.freqFunc] << "rigstate: vfoMode:" << rigState.vfoMode << "vfo" << rigState.vfo << "rx" << rigState.receiver;
+    return cmd;
+}
+
 queuePriority cachingQueue::del(funcs func, uchar receiver)
 {
     // This will immediately delete any matching commands.
@@ -329,7 +348,7 @@ void cachingQueue::receiveValue(funcs func, QVariant value, uchar receiver)
     if (mutex.tryLock(CACHE_LOCK_TIME)) {
         cacheItem c = cacheItem(func,value,receiver);
         items.enqueue(c);
-        updateCache(true,func,value,receiver);
+        updateCache(true,func,value,receiver);        
         mutex.unlock();
 #ifndef Q_OS_MACOS
         waiting.wakeOne();
@@ -343,6 +362,36 @@ void cachingQueue::receiveValue(funcs func, QVariant value, uchar receiver)
 void cachingQueue::updateCache(bool reply, queueItem item)
 {
     // Mutex MUST be locked by the calling function.
+
+    // We need to make sure that all "main" frequencies/modes are updated.
+    if (reply)
+    {
+        // Is this a command that might have updated our state?
+        if (item.command == funcSatelliteMode && item.param.value<bool>())
+            rigState.vfoMode=vfoModeType_t::vfoModeSat;
+        if (item.command == funcMemoryMode && item.param.value<bool>())
+            rigState.vfoMode=vfoModeType_t::vfoModeMem;
+        if (item.command == funcVFOMode && item.param.value<bool>())
+            rigState.vfoMode=vfoModeType_t::vfoModeVfo;
+        if (item.command == funcVFOBandMS)
+            rigState.receiver = item.param.value<uchar>();
+    } else {
+        // If we are requesting a particular VFO, set our state as the rig will not reply
+        if (item.command == funcSelectVFO) {
+            rigState.vfo = item.param.value<vfo_t>();
+        } else if (item.command == funcVFOASelect) {
+            rigState.vfo = vfo_t::vfoA;
+        } else if (item.command == funcVFOBSelect && rigCaps->numVFO > 1) {
+            rigState.vfo = vfo_t::vfoB;
+        } else if (item.command == funcVFOMainSelect) {
+            rigState.vfo = vfo_t::vfoMain;
+            rigState.receiver=0;
+        } else if (item.command == funcVFOSubSelect && rigCaps->numReceiver > 1) {
+            rigState.vfo = vfo_t::vfoSub;
+            rigState.receiver=1;
+        }
+    }
+
     auto cv = cache.find(item.command);
     while (cv != cache.end() && cv->command == item.command) {
         if (cv->receiver == item.receiver) {
@@ -370,6 +419,7 @@ void cachingQueue::updateCache(bool reply, queueItem item)
     cacheItem c;
     c.command = item.command;
     c.receiver = item.receiver;
+
     if (reply) {
         c.reply = QDateTime::currentDateTime();
     } else {
@@ -377,9 +427,11 @@ void cachingQueue::updateCache(bool reply, queueItem item)
     }
     // If we are sending an actual value, update the cache with it
     // Value will be replaced if invalid on next get()
-    if (item.param.isValid()) {
-        c.value = item.param;
+    if (item.param.isValid())
+    {
+        c.value.setValue(item.param);
     }
+
     cache.insert(item.command,c);
 
 }
@@ -407,9 +459,9 @@ cacheItem cachingQueue::getCache(funcs func, uchar receiver)
     }
     // If the cache is more than 5-20 seconds old, re-request it as it may be stale (maybe make this a config option?)
     // Using priorityhighest WILL slow down the S-Meter when a command intensive client is connected to rigctl
-    if (func != funcNone && func != funcPowerControl && (!ret.value.isValid() || ret.reply.addSecs(QRandomGenerator::global()->bounded(5,20)) <= QDateTime::currentDateTime())) {
+    if (func != funcNone && func != funcPowerControl && func != funcSelectVFO && (!ret.value.isValid() || ret.command == funcSWRMeter || ret.reply.addSecs(QRandomGenerator::global()->bounded(5,20)) <= QDateTime::currentDateTime())) {
         //qInfo() << "No (or expired) cache found for" << funcString[func] << "requesting";
-        add(priorityHighest,func,false,receiver);
+        add(priorityImmediate,func,false,receiver);
     }
     return ret;
 }
@@ -453,6 +505,9 @@ bool cachingQueue::compare(QVariant a, QVariant b)
         } else if (!strcmp(a.typeName(),"int")) {
             if (a.value<int>() != b.value<int>())
                 changed=true;
+        } else if (!strcmp(a.typeName(),"double")) {
+            if (a.value<double>() != b.value<double>())
+                changed=true;
         } else if (!strcmp(a.typeName(),"modeInfo")) {
             if (a.value<modeInfo>().mk != b.value<modeInfo>().mk || a.value<modeInfo>().reg != b.value<modeInfo>().reg
                 || a.value<modeInfo>().filter != b.value<modeInfo>().filter || a.value<modeInfo>().data != b.value<modeInfo>().data) {
@@ -480,8 +535,16 @@ bool cachingQueue::compare(QVariant a, QVariant b)
             if (a.value<meter_t>() != b.value<meter_t>())
                 changed=true;
         } else if (!strcmp(a.typeName(),"vfo_t")) {
-                if (a.value<vfo_t>() != b.value<vfo_t>())
-                    changed=true;
+            if (a.value<vfo_t>() != b.value<vfo_t>())
+                changed=true;
+        } else if (!strcmp(a.typeName(),"lpfhpf")) {
+            if (a.value<lpfhpf>().lpf != b.value<lpfhpf>().lpf || a.value<lpfhpf>().hpf != b.value<lpfhpf>().hpf  )
+                changed=true;
+        } else if (!strcmp(a.typeName(),"spectrumBounds")) {
+            if (a.value<spectrumBounds>().edge != b.value<spectrumBounds>().edge ||
+                    a.value<spectrumBounds>().start != b.value<spectrumBounds>().start ||
+                    a.value<spectrumBounds>().end != b.value<spectrumBounds>().end  )
+                changed=true;
         } else if (!strcmp(a.typeName(),"centerSpanData")) {
             if (a.value<centerSpanData>().cstype != b.value<centerSpanData>().cstype || a.value<centerSpanData>().freq != b.value<centerSpanData>().freq  )
                 changed=true;
@@ -492,7 +555,7 @@ bool cachingQueue::compare(QVariant a, QVariant b)
                 changed=true;
         } else if (!strcmp(a.typeName(),"scopeData") || !strcmp(a.typeName(),"memoryType")
                    || !strcmp(a.typeName(),"bandStackType")  || !strcmp(a.typeName(),"timekind") || !strcmp(a.typeName(),"datekind")
-                   || !strcmp(a.typeName(),"meterkind")) {
+                   || !strcmp(a.typeName(),"meterkind") || !strcmp(a.typeName(),"udpPreferences")) {
             changed=true; // Always different
         } else {
             // Maybe Try simple comparison?
