@@ -57,7 +57,7 @@ static const subCommandStruct levels_str[] =
     {"AF",funcAfGain,typeFloat},
     {"RF",funcRfGain,typeFloat},
     {"SQL",funcSquelch,typeFloat},
-    {"IF",funcIFFilterWidth,typeFloat},
+    {"IF",funcFilterWidth,typeFloat},
     {"APF",funcAPFLevel,typeFloat},
     {"NR",funcNRLevel,typeFloat},
     {"PBT_IN",funcPBTInner,typeFloat},
@@ -68,7 +68,7 @@ static const subCommandStruct levels_str[] =
     {"KEYSPD",funcKeySpeed,typeUShort},
     {"NOTCHF",funcNotchFilter,typeUChar},
     {"COMP",funcCompressorLevel,typeFloat},
-    {"AGC",funcAGCTime,typeUChar},
+    {"AGC",funcAGCTimeConstant,typeUChar},
     {"BKINDL",funcBreakInDelay,typeFloat},
     {"BAL",funcNone,typeFloat},
     {"METER",funcNone,typeFloat},
@@ -78,13 +78,13 @@ static const subCommandStruct levels_str[] =
     {"SLOPE_HIGH",funcNone,typeFloat},
     {"BKIN_DLYMS",funcBreakInDelay,typeFloat},
     {"RAWSTR",funcNone,typeFloat},
-    {"SWR",funcSMeter,typeSWR},
-    {"ALC",funcALCMeter,typeFloat},
-    {"STRENGTH",funcSMeter,typeFloat},
-    {"RFPOWER_METER",funcPowerMeter,typeFloat},
-    {"COMPMETER",funcCompMeter,typeFloat},
-    {"VD_METER",funcVdMeter,typeFloat},
-    {"ID_METER",funcIdMeter,typeFloat},
+    {"SWR",funcSWRMeter,typeSWR},
+    {"ALC",funcALCMeter,typeDouble},
+    {"STRENGTH",funcSMeter,typeDouble},
+    {"RFPOWER_METER",funcPowerMeter,typeDouble},
+    {"COMPMETER",funcCompMeter,typeDouble},
+    {"VD_METER",funcVdMeter,typeDouble},
+    {"ID_METER",funcIdMeter,typeDouble},
     {"NOTCHF_RAW",funcNone,typeFloat},
     {"MONITOR_GAIN",funcMonitorGain,typeFloat},
     {"NQ",funcNone,typeFloat},
@@ -146,7 +146,7 @@ static const subCommandStruct params_str[] =
 {
     {"ANN",funcNone,typeUChar},
     {"APO",funcNone,typeUChar},
-    {"BACKLIGHT",funcBackLight,typeUChar},
+    {"BACKLIGHT",funcBackLightLevel,typeUChar},
     {"BEEP",funcNone,typeUChar},
     {"TIME",funcTime,typeUChar},
     {"BAT",funcNone,typeUChar},
@@ -211,18 +211,18 @@ static const commandStruct commands_list[] =
     { 0x93, "get_dcs_sql",      funcCSQLCode,           typeUChar,    ARG_OUT, "DCS Sql" },
     { 'V',  "set_vfo",          funcSelectVFO,          typeVFO,    ARG_IN  | ARG_NOVFO, "VFO" },
     { 'v',  "get_vfo",          funcSelectVFO,          typeVFO,    ARG_NOVFO | ARG_OUT, "VFO" },
-    { 'T',  "set_ptt",          funcTransceiverStatus,  typeBinary,    ARG_IN, "PTT" },
-    { 't',  "get_ptt",          funcTransceiverStatus,  typeBinary,    ARG_OUT, "PTT" },
+    { 'T',  "set_ptt",          funcTransceiverStatus,  typeBinary,    ARG_IN, "VFO", "PTT" },
+    { 't',  "get_ptt",          funcTransceiverStatus,  typeBinary,    ARG_OUT, "VFO", "PTT" },
     { 'E',  "set_mem",          funcMemoryContents,     typeMode,    ARG_IN, "Memory#" },
     { 'e',  "get_mem",          funcMemoryContents,     typeMode,    ARG_OUT, "Memory#" },
     { 'H',  "set_channel",      funcMemoryMode,         typeUChar,    ARG_IN  | ARG_NOVFO, "Channel"},
     { 'h',  "get_channel",      funcMemoryMode,         typeUChar,    ARG_IN  | ARG_NOVFO, "Channel", "Read Only" },
     { 'B',  "set_bank",         funcMemoryGroup,        typeUChar,    ARG_IN, "Bank" },
     { '_',  "get_info",         funcNone,               typeUChar,    ARG_OUT | ARG_NOVFO, "Info" },
-    { 'J',  "set_rit",          funcRITFreq,            typeShort,    ARG_IN, "RIT" },
-    { 'j',  "get_rit",          funcRITFreq,            typeShort,    ARG_OUT, "RIT" },
-    { 'Z',  "set_xit",          funcNone,               typeShort,    ARG_IN, "XIT" },
-    { 'z',  "get_xit",          funcNone,               typeShort,    ARG_OUT, "XIT" },
+    { 'J',  "set_rit",          funcRitFreq,            typeShort,    ARG_IN, "RIT" },
+    { 'j',  "get_rit",          funcRitFreq,            typeShort,    ARG_OUT, "RIT" },
+    { 'Z',  "set_xit",          funcXitFreq,            typeShort,    ARG_IN, "XIT" },
+    { 'z',  "get_xit",          funcXitFreq,            typeShort,    ARG_OUT, "XIT" },
     { 'Y',  "set_ant",          funcAntenna,            typeUChar,    ARG_IN, "Antenna", "Option" },
     { 'y',  "get_ant",          funcAntenna,            typeUChar,    ARG_IN1 | ARG_OUT2 | ARG_NOVFO, "AntCurr", "Option", "AntTx", "AntRx" },
     { 0x87, "set_powerstat",    funcPowerControl,       typeBinary,    ARG_IN  | ARG_NOVFO, "Power Status" },
@@ -247,7 +247,7 @@ static const commandStruct commands_list[] =
     { '1',  "dump_caps",        funcNone,               typeUChar,    ARG_NOVFO },
     { '3',  "dump_conf",        funcNone,               typeUChar,    ARG_NOVFO },
     { 0x8f, "dump_state",       funcNone,               typeUChar,    ARG_OUT | ARG_NOVFO },
-    { 0xf0, "chk_vfo",          funcSelectVFO,          typeUChar,    ARG_NOVFO, "ChkVFO" },   /* rigctld only--check for VFO mode */
+    { 0xf0, "chk_vfo",          funcNone,               typeUChar,    ARG_NOVFO, "ChkVFO" },   /* rigctld only--check for VFO mode */
     { 0xf2, "set_vfo_opt",      funcNone,               typeUChar,    ARG_NOVFO | ARG_IN, "Status" }, /* turn vfo option on/off */
     { 0xf3, "get_vfo_info",     funcSelectVFO,          typeVFOInfo,    ARG_IN1 | ARG_NOVFO | ARG_OUT5, "VFO", "Freq", "Mode", "Width", "Split", "SatMode" }, /* get several vfo parameters at once */
     { 0xf5, "get_rig_info",     funcNone,               typeUChar,    ARG_NOVFO | ARG_OUT, "RigInfo" }, /* get several vfo parameters at once */
@@ -341,10 +341,7 @@ rigCtlClient::rigCtlClient(int socketId, rigCtlD* parent) : QObject(parent)
     connect(parent, SIGNAL(sendData(QString)), this, SLOT(sendData(QString)), Qt::DirectConnection);
     qInfo(logRigCtlD()) << " session connected: " << sessionId;
 
-    currentVfo = vfoUnknown;
-
     // Find what VFOs we have:
-
     if (rigCaps->numVFO > 0 && rigCaps->commands.contains(funcVFOASelect))
         vfoList |= 1<<0;
     if (rigCaps->numVFO > 1 && rigCaps->commands.contains(funcVFOBSelect))
@@ -355,8 +352,6 @@ rigCtlClient::rigCtlClient(int socketId, rigCtlD* parent) : QObject(parent)
         vfoList |= 1<<25;
     if (rigCaps->commands.contains(funcMemoryMode))
         vfoList |= 1<<28;
-
-    swrCal = ICOM_SWR_CAL;
 }
 
 void rigCtlClient::socketReadyRead()
@@ -374,10 +369,12 @@ void rigCtlClient::socketReadyRead()
     QString sep = "\n";
 
     int ret = -RIG_EINVAL;
-
+    bool found = false;
     bool setCommand = false;
     bool longCommand = false;
     bool extended = false;
+    bool sendStatus = true;
+
     QStringList response;
 
     for (QString &commands : commandList)
@@ -433,14 +430,16 @@ void rigCtlClient::socketReadyRead()
 
         QStringList command = commands.split(" ");
         command.removeAll({}); // Remove any empty strings (double-whitespace issue)
+        found=false;
+        sendStatus = true;
         for (int i=0; commands_list[i].sstr != 0x00; i++)
         {
             if ((longCommand && !strncmp(command[0].toLocal8Bit(), commands_list[i].str,MAXNAMESIZE)) ||
                    (!longCommand && !commands.isNull() && commands[0].toLatin1() == commands_list[i].sstr))
             {
                 command.removeFirst(); // Remove the actual command so it only contains params
-
-                if (extended){
+                found = true;
+                if (extended && commands_list[i].sstr != 0xf0){
                     // First we need to repeat the original command back:
                     QString repeat=QString("%1:").arg(commands_list[i].str);
                     for( const auto &c: command)
@@ -506,8 +505,10 @@ void rigCtlClient::socketReadyRead()
                 else if (commands_list[i].sstr == 0xf0)
                 {
                     chkVfoEecuted = true;
-                    response.append(QString("CHKVFO %0").arg(uchar(1)));
+                    response.append(QString("ChkVFO: %0").arg(uchar(0)));
                     ret = RIG_OK;
+                    // chk_vfo doesn't output RPRT
+                    sendStatus=false;
                     break;
                 }
                 // Special commands are funcNone so will not get called here
@@ -523,7 +524,7 @@ void rigCtlClient::socketReadyRead()
             break;  // Cannot be a compound command so just output result.
         }
 
-        if (!longCommand && !commands.isEmpty()){
+        if (!longCommand && !commands.isEmpty() ){
             commands.remove(0,1);
             goto restart; // Need to restart loop without going to next command incase of compound command
         }
@@ -537,15 +538,12 @@ void rigCtlClient::socketReadyRead()
     for (int i = 0;i<response.size();i++)
     {
         if (!response[i].isEmpty()) {
-            sendData(QString("%0%1").arg(response[i],(i<response.size()-1)?sep:""));
+            sendData(QString("%0%1").arg(response[i],sep));
         }
     }
 
-    if (ret < 0 || setCommand || extended)
-        sendData(QString("%0RPRT %1\n").arg(response.isEmpty()?"":sep).arg(QString::number(ret)));
-    else if (sep == '\n')
-        sendData("\n");
-
+    if (found && sendStatus && (ret < 0 || setCommand || extended))
+        sendData(QString("RPRT %1\n").arg(QString::number(ret)));
 }
 
 void rigCtlClient::socketDisconnected()
@@ -684,7 +682,8 @@ quint64 rigCtlClient::getRadioModes(QString md)
         for (int i = 0; mode_str[i].str[0] != '\0'; i++)
         {
             QString mstr = QString(mode_str[i].str);
-            if (rigCaps->hasDataModes) {
+            if (rigCaps->inputs.size())
+            {
                 if (mstr.contains(mode.name))
                 {
                     // qDebug(logRigCtlD()) << "Found data mode:" << mode.name << mode_str[i].mode;
@@ -747,15 +746,20 @@ quint8 rigCtlClient::antFromName(QString name) {
 
 vfo_t rigCtlClient::vfoFromName(QString vfo) {
 
+    rigStateType state = queue->getState();
     vfo_t v = vfoUnknown;
-    if (vfo.toUpper() == "VFOA")
+    if (vfo.toUpper() == "CURRVFO")
+    {
+        v=state.vfo;
+    }
+    else if (vfo.toUpper() == "VFOA")
     {
         if (rigCaps->commands.contains(funcVFOASelect))
             v = vfoA;
         else if (rigCaps->commands.contains(funcVFOMainSelect))
             v = vfoMain;
         else
-            v = vfoCurrent;
+            v = state.vfo;
     }
     else if (vfo.toUpper() == "VFOB")
     {
@@ -764,7 +768,7 @@ vfo_t rigCtlClient::vfoFromName(QString vfo) {
         else if (rigCaps->commands.contains(funcVFOSubSelect))
             v = vfoSub;
         else
-            v = vfoCurrent;
+            v = state.vfo;
     }
     else if (vfo.toUpper() == "MAIN")
     {
@@ -773,7 +777,7 @@ vfo_t rigCtlClient::vfoFromName(QString vfo) {
         else if (rigCaps->commands.contains(funcVFOASelect))
             v = vfoA;
         else
-            v = vfoCurrent;
+            v = state.vfo;
     }
     else if (vfo.toUpper() == "SUB")
     {
@@ -782,11 +786,14 @@ vfo_t rigCtlClient::vfoFromName(QString vfo) {
         else if (rigCaps->commands.contains(funcVFOBSelect))
             v = vfoB;
         else
-            v = vfoCurrent;
+            v = state.vfo;
     }
     else if (vfo.toUpper() == "MEM")
     {
-        v = vfoMem;
+        if (rigCaps->commands.contains(funcMemoryMode))
+            v = vfoMem;
+        else
+            v = state.vfo;
     }
 
     //qInfo() << "vfoFromName" << vfo << "returns" << v;
@@ -825,38 +832,6 @@ QString rigCtlClient::getVfoName(vfo_t vfo)
     return ret;
 }
 
-int rigCtlClient::getCalibratedValue(quint8 meter,cal_table_t cal) {
-    
-    int interp;
-
-    int i = 0;
-    for (i = 0; i < cal.size; i++) {
-        if (meter < cal.table[i].raw)
-        {
-            break;
-        }
-    }
-
-    if (i == 0)
-    {
-        return cal.table[0].val;
-    } 
-    else if (i >= cal.size)
-    {
-        return cal.table[i - 1].val;
-    } 
-    else if (cal.table[i].raw == cal.table[i - 1].raw)
-    {
-        return cal.table[i].val;
-    }
-
-    interp = ((cal.table[i].raw - meter)
-        * (cal.table[i].val - cal.table[i - 1].val))
-        / (cal.table[i].raw - cal.table[i - 1].raw);
-
-    return cal.table[i].val - interp;
-}
-
 unsigned long rigCtlClient::doCrc(quint8* p, size_t n)
 {
     unsigned long crc = 0xfffffffful;
@@ -901,6 +876,8 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
     // This is a main command
     int ret = -RIG_EINVAL;
     funcs func = cmd.func;
+    rigStateType state=queue->getState();
+
 
     if (rigCaps == Q_NULLPTR)
         return ret;
@@ -934,25 +911,31 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
             val.setValue(static_cast<ushort>(float(params[0].toFloat() * 5.1)));
             break;
         case typeBinary:
-            val.setValue(static_cast<bool>(params[0].toInt()));
+            if (params.length()>1){
+                queue->add(priorityImmediate, queueItem(funcSelectVFO, QVariant::fromValue<vfo_t>(vfoFromName(params[0])),false,state.receiver));
+                val.setValue(static_cast<bool>(params[1].toInt()));
+            } else {
+                val.setValue(static_cast<bool>(params[0].toInt()));
+            }
             break;
         case typeFreq:
-        {
+        {            
             freqt f;
-            currentRx=0;
             if (cmd.sstr == 'I') {
-                func = getFreqFunc(splitVfo,true);
-                if (splitVfo == vfoSub && rigCaps->numReceiver > 1)
-                    currentRx=1;
-            } else {
-                func = getFreqFunc(currentVfo,true);
+                state.vfo=splitVfo;
+                state.receiver = uchar((rigCaps->numVFO == 1 && rigCaps->numReceiver>1));
             }
 
-            //queue->add(priorityImmediate, queueItem(funcSelectVFO, QVariant::fromValue<vfo_t>(currentVfo),false,currentRx));
+            if (params.length() > 1) {                
+                state.vfo = vfoFromName(params[0]);
+                f.Hz = static_cast<quint64>(params[1].toDouble());
+            } else {
+                f.Hz = static_cast<quint64>(params[0].toDouble());
+            }
 
-            f.Hz = static_cast<quint64>(params[0].toFloat());
+            func = queue->getVfoCommand(state.vfo,state.receiver,true).freqFunc;
             f.VFO = activeVFO;
-            f.MHzDouble = f.Hz/1000000.0;
+            f.MHzDouble = static_cast<double>(f.Hz/1000000.0);
             val.setValue(f);
             break;
         }
@@ -967,32 +950,43 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
 
         case typeMode:
         {
-            currentRx=0;
             if (cmd.sstr == 'X') {
-                func = getModeFunc(splitVfo,true);
-                if (splitVfo == vfoSub && rigCaps->numReceiver > 1)
-                    currentRx=1;
-            } else {
-                func = getModeFunc(currentVfo,true);
+                state.vfo=splitVfo;
+                state.receiver = uchar((rigCaps->numVFO==1 && rigCaps->numReceiver>1));
             }
-            //queue->add(priorityImmediate, queueItem(funcSelectVFO, QVariant::fromValue<vfo_t>(currentVfo),false,currentRx));
 
             modeInfo mi;
-            if (getMode(params[0],mi))
+            QString mode="";
+            QString width="";
+
+            if (params.size() == 3)
+            {
+                state.vfo=vfoFromName(params[0]);
+                mode=params[1];
+                width=params[2];
+            }
+            else if (params.size() == 2)
+            {
+                mode=params[0];
+                width=params[1];
+            }
+
+            // We have VFO, Mode and PB
+            func = queue->getVfoCommand(state.vfo,state.receiver,true).modeFunc;
+
+            bool ok;
+            int pb = width.toInt(&ok);
+            if (ok && pb>0) {
+                queue->add(priorityImmediate, queueItem(funcFilterWidth, QVariant::fromValue<ushort>(pb),false,state.receiver));
+            }
+
+            if (getMode(mode,mi))
             {
                 val.setValue(mi);
             } else {
                 qInfo(logRigCtlD()) << "Mode not found:" << params[0];
                 return -RIG_EINVAL;
             }
-            if (params.size() > 1) {
-                // We have a passband as well
-                bool ok;
-                int pb = params[1].toInt(&ok);
-                if (ok && pb>0) {
-                    queue->add(priorityImmediate, queueItem(funcFilterWidth, QVariant::fromValue<ushort>(pb),false,currentRx));
-                }
-            }
             break;
         }
         case typeVFO:
@@ -1015,12 +1009,8 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
             }
             else
             {
-                currentVfo = vfoFromName(params[0]);
-                val.setValue(currentVfo);
-                if (currentVfo == vfoSub && rigCaps->numReceiver > 1)
-                    currentRx = 1;
-                else
-                    currentRx = 0;
+                state.vfo = vfoFromName(params[0]);
+                val.setValue(state.vfo);
             }
             break;
         case typeString:
@@ -1039,7 +1029,7 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
             return -RIG_EINVAL;
         }
 
-        queue->add(priorityImmediate, queueItem(func, val,false,currentRx));
+        queue->add(priorityImmediate, queueItem(func, val,false,state.receiver));
 
     } else {
         // Simple get command
@@ -1048,38 +1038,39 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
         // Build QStringList of Prefixes. This is a bit messy, but not sure how else do to it?
         QStringList prefixes=buildPrefixes(cmd,extended);
 
-        if (cmd.sstr=='i') {
-            func = getFreqFunc(splitVfo);
-            if (splitVfo == vfoSub && rigCaps->numReceiver > 1)
-                currentRx=1;
-
+        if (!prefixes.length())
+        {
+            qWarning(logRigCtlD()) << "No prefixes found for cmd" << cmd.str << "using func" << funcString[func] << "aborting";
         }
-        else if (cmd.sstr=='x') {
-            func = getModeFunc(splitVfo);
-            if (splitVfo == vfoSub && rigCaps->numReceiver > 1)
-                currentRx=1;
-        } else {
 
-            if (func == funcFreqGet) {
-                func = getFreqFunc(currentVfo);
-            } else if (func == funcModeGet) {
-                func = getModeFunc(currentVfo);
+        if (cmd.sstr=='i' ||  cmd.sstr=='x') {
+            if (splitVfo == vfoSub && rigCaps->numReceiver > 1) {
+                state.receiver=1;
             }
+            state.vfo = splitVfo;
         }
 
-        //qInfo() << "getting Cache Value for func" << funcString[func] << "on rx" << currentRx;
+        if (func == funcFreqGet) {
+            func = queue->getVfoCommand(state.vfo,state.receiver,true).freqFunc;
+        } else if (func == funcModeGet) {
+            func = queue->getVfoCommand(state.vfo,state.receiver,true).modeFunc;
+        }
+
+        //qInfo() << "getting Cache Value for func" << funcString[func] << "on rx" << state.receiver;
         if (rigCaps->commands.contains(func))
-            item = queue->getCache(func,currentRx);
+            item = queue->getCache(func,state.receiver);
+
+        if (prefixes.length() && params.length())
+        {
+            response.append(QString("%0%1").arg(prefixes[0], params[0]));
+        }
 
         ret = RIG_OK;
         switch (cmd.type){
         case typeBinary:
         {
             bool b = item.value.toBool();
-            if (prefixes.length())
-            {
-                response.append(QString("%0%1").arg(prefixes[0], QString::number(b)));
-            }
+            response.append(QString("%0%1").arg(prefixes[prefixes.length()-1], QString::number(b)));
             break;
         }
         case typeUChar:
@@ -1091,57 +1082,37 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
             if (cmd.func == funcLockFunction)
                 i = modeLock;
 
-            if (prefixes.length())
-            {
-                response.append(QString("%0%1").arg(prefixes[0], QString::number(i)));
-            }
+            response.append(QString("%0%1").arg(prefixes[prefixes.length()-1], QString::number(i)));
             break;
         }
         case typeFloat:
         {
             float f = item.value.toFloat() / 255.0;
-            if (prefixes.length())
-            {
-                response.append(QString("%0%1").arg(prefixes[0],QString::number(f,typeFloat,6)));
-            }
+            response.append(QString("%0%1").arg(prefixes[prefixes.length()-1],QString::number(f,typeFloat,6)));
             break;
         }
         case typeFloatDiv:
         {
             float f = item.value.toFloat() * 10;
-            if (prefixes.length())
-            {
-                response.append(QString("%0%1").arg(prefixes[0],QString::number(f,typeFloat,6)));
-            }
+            response.append(QString("%0%1").arg(prefixes[prefixes.length()-1],QString::number(f,typeFloat,6)));
             break;
         }
         case typeFloatDiv5:
         {
             float f = item.value.toFloat() / 5.1;
-            if (prefixes.length())
-            {
-                response.append(QString("%0%1").arg(prefixes[0],QString::number(f,typeFloat,6)));
-            }
+            response.append(QString("%0%1").arg(prefixes[prefixes.length()-1],QString::number(f,typeFloat,6)));
             break;
         }
         case typeFreq:
         { // Frequency
             freqt f = item.value.value<freqt>();
-
-            //qInfo(logRigCtlD()) << "Got Freq" << funcString[func] << f.Hz << "on VFO" << f.VFO;
-            if (prefixes.length())
-            {
-                response.append(QString("%0%1").arg(prefixes[0],QString::number(f.Hz)));
-            }
+            response.append(QString("%0%1.000000").arg(prefixes[prefixes.length()-1],QString::number(f.Hz)));
             break;
         }
         case typeVFO:
         { // VFO
             //uchar v = item.value.value<uchar>();
-            if (prefixes.length())
-            {
-                response.append(QString("%0%1").arg(prefixes[0],getVfoName(currentVfo)));
-            }
+            response.append(QString("%0%1").arg(prefixes[prefixes.length()-1],getVfoName(state.vfo)));
             break;
         }
         case typeSplitVFO:
@@ -1156,31 +1127,30 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
 
         case typeVFOInfo:
         {
-            uchar rx = 0;
-            funcs freqFunc = getFreqFunc(vfoFromName(params[0]));
-            funcs modeFunc = getModeFunc(vfoFromName(params[0]));
+            funcs freqFunc = queue->getVfoCommand(state.vfo,state.receiver,true).freqFunc;
+            funcs modeFunc = queue->getVfoCommand(state.vfo,state.receiver,true).modeFunc;
 
             if (rigCaps->numReceiver > 1 && params.size() && params[0] == "Sub") {
-                rx=1;
+                state.receiver=1;
             }
 
             if (prefixes.length()>1)
-                response.append(QString("%0%1").arg(prefixes[1],QString::number(queue->getCache(freqFunc,rx).value.value<freqt>().Hz)));
+                response.append(QString("%0%1").arg(prefixes[1],QString::number(queue->getCache(freqFunc,state.receiver).value.value<freqt>().Hz)));
             if (prefixes.length()>2)
-                response.append(QString("%0%1").arg(prefixes[2],getMode(queue->getCache(modeFunc,rx).value.value<modeInfo>())));
+                response.append(QString("%0%1").arg(prefixes[2],getMode(queue->getCache(modeFunc,state.receiver).value.value<modeInfo>())));
             if (prefixes.length()>3)
-                response.append(QString("%0%1").arg(prefixes[3],QString::number(queue->getCache(funcFilterWidth,rx).value.value<ushort>())));
-            if (prefixes.length()>4)
-                response.append(QString("%0%1").arg(prefixes[4],QString::number(queue->getCache(funcSplitStatus,rx).value.value<duplexMode_t>())));
+                response.append(QString("%0%1").arg(prefixes[3],QString::number(queue->getCache(funcFilterWidth,state.receiver).value.value<ushort>())));
+            if (prefixes.length()>4) /* Split is only ever on the first rx */
+                response.append(QString("%0%1").arg(prefixes[4],QString::number(queue->getCache(funcSplitStatus,0).value.value<duplexMode_t>())));
             if (prefixes.length()>5)
-                response.append(QString("%0%1").arg(prefixes[5],QString::number(queue->getCache(funcSatelliteMode,rx).value.value<bool>())));
+                response.append(QString("%0%1").arg(prefixes[5],QString::number(queue->getCache(funcSatelliteMode,state.receiver).value.value<bool>())));
             break;
         }
 
         case typeMode:
         { // Mode
             modeInfo m = item.value.value<modeInfo>();
-            cacheItem f = queue->getCache(funcFilterWidth,currentRx);
+            cacheItem f = queue->getCache(funcFilterWidth,state.receiver);
             if (prefixes.length() > 0)
                 response.append(QString("%0%1").arg(prefixes[0],getMode(m)));
             if (prefixes.length() > 1)
@@ -1191,7 +1161,6 @@ int rigCtlClient::getCommand(QStringList& response, bool extended, const command
         {
             // Stop sending CW if a blank command is received.
             queue->add(priorityImmediate, queueItem(func, QString(QChar(0xff)),false,0));
-            ret = RIG_OK;
             break;
         }
         default:
@@ -1214,6 +1183,7 @@ int rigCtlClient::getSubCommand(QStringList& response, bool extended, const comm
     if (rigCaps == Q_NULLPTR)
         return ret;
 
+    rigStateType state = queue->getState();
 
     QString resp;
     // Get the required subCommand and process it
@@ -1280,73 +1250,55 @@ int rigCtlClient::getSubCommand(QStringList& response, bool extended, const comm
                         return -RIG_EINVAL;
                     }
                     if (rigCaps->commands.contains(sub[i].func))
-                        queue->add(priorityImmediate, queueItem(sub[i].func, val,false,currentRx));
+                        queue->add(priorityImmediate, queueItem(sub[i].func, val,false,state.receiver));
                 } else if (params.size() == 1){
                     // Not expecting a second argument as it is a get so dump the cache
                     cacheItem item;
                     if (rigCaps->commands.contains(sub[i].func))
-                        item = queue->getCache(sub[i].func,currentRx);
+                        item = queue->getCache(sub[i].func,state.receiver);
                     int val = 0;
-                    // Special situation for S-Meter
-                    if (params[0] == "STRENGTH") {
-                        if (rigCaps->model == model7610)
-                            val = getCalibratedValue(item.value.toUInt(), IC7610_STR_CAL);
-                        else if (rigCaps->model == model7850)
-                            val = getCalibratedValue(item.value.toUInt(), IC7850_STR_CAL);
-                        else
-                            val = getCalibratedValue(item.value.toUInt(), IC7300_STR_CAL);
-                        resp.append(QString("%1").arg(val));
-                    }
-                    else
+
+                    switch (sub[i].type)
                     {
-                        switch (sub[i].type)
-                        {
-                        case typeBinary:
-                            resp.append(QString("%1").arg(item.value.toBool()));
-                            break;
-                        case typeUChar:
-                        {
-                            int val = item.value.toInt();
-                            if (params[0] == "FBKIN")
-                                val = (val >> 1) & 0x01;
-                            if (params[0] == "AGC")
-                                val = (val >> 1);
-                            resp.append(QString::number(val));
-                            break;
-                        }
-                        case typeSWR:{
-                            float f=rawToFloat(item.value.toInt(),&swrCal);
-                            resp.append(QString::number(f,'f',6));
-                            break;
-                        }
-                        case typeUShort:
-                        case typeShort:
-                            resp.append(QString::number(item.value.toInt()));
-                            break;
-                        case typeFloat:
-                            resp.append(QString::number(item.value.toFloat() / 255.0,'f',6));
-                            break;
-                        case typeFloatDiv:
-                            resp.append(QString::number(item.value.toFloat() * 10.0,'f',6));
-                            break;
-                        case typeFloatDiv5:
-                            resp.append(QString::number(item.value.toFloat()/5.1,'f',6));
-                            break;
-                        default:
-                            qInfo(logRigCtlD()) << "Unhandled:" << item.value.toUInt() << "OUT" << val;
-                            ret = -RIG_EINVAL;
-                        }
+                    case typeBinary:
+                        resp.append(QString("%1").arg(item.value.toBool()));
+                        break;
+                    case typeUChar:
+                    {
+                        int val = item.value.toInt();
+                        if (params[0] == "FBKIN")
+                            val = (val >> 1) & 0x01;
+                        if (params[0] == "AGC")
+                            val = (val >> 1);
+                        resp.append(QString::number(val));
+                        break;
                     }
-                    //if (extended && cmd.arg2 != NULL)
-                    //{
-                    //    response.append(QString("%0: %1").arg(cmd.str, params[0]));
-                    //    response.append(QString("%0: %1").arg(cmd.arg2, resp));
-                    //}
-                    //else
-                    //{
-                        response.append(resp);
-                    //}
-                } else {
+                    case typeSWR:
+                    case typeDouble:
+                        resp.append(QString::number(item.value.toDouble(),'f',6));
+                        break;
+                    case typeUShort:
+                    case typeShort:
+                        resp.append(QString::number(item.value.toInt()));
+                        break;
+                    case typeFloat:
+                        resp.append(QString::number(item.value.toFloat() / 255.0,'f',6));
+                        break;
+                    case typeFloatDiv:
+                        resp.append(QString::number(item.value.toFloat() * 10.0,'f',6));
+                        break;
+                    case typeFloatDiv5:
+                        resp.append(QString::number(item.value.toFloat()/5.1,'f',6));
+                        break;
+                    default:
+                        qInfo(logRigCtlD()) << funcString[sub[i].func] << "Unhandled (" << item.value.typeName() << ")" << item.value.toUInt() << "OUT" << val;
+                        ret = -RIG_EINVAL;
+                    }
+                    qDebug(logRigCtlD()) << "Sending " << funcString[sub[i].func] <<  "data:" << resp;
+                    response.append(resp);
+                }
+                else
+                {
                     qInfo(logRigCtlD()) << "Invalid number of params" <<  params.size();
                     ret = -RIG_EINVAL;
                 }
@@ -1461,9 +1413,9 @@ int rigCtlClient::dumpState(QStringList &response, bool extended)
     if(rigCaps->commands.contains(funcAttenuator)) {
         for (auto att : rigCaps->attenuators)
         {
-            if (att == 0)
+            if (att.num == 0)
                 continue;
-            attens.append(QString(" %1").arg(att));
+            attens.append(QString(" %1").arg(att.num));
         }
         if (attens.endsWith(" "))
             attens.chop(1);
@@ -1547,63 +1499,6 @@ int rigCtlClient::dumpCaps(QStringList &response, bool extended)
     return RIG_OK;
 }
 
-funcs rigCtlClient::getFreqFunc(vfo_t vfo, bool set)
-{
-    funcs func = ((rigCaps->commands.contains(funcFreq)) ? funcFreq: funcFreqGet) ;
-
-    currentRx = 0;
-    if (set)
-        func = ((rigCaps->commands.contains(funcFreq)) ? funcFreq: funcFreqSet) ;
-
-    if ((vfo == vfoA && currentVfo == vfoA) || (vfo == vfoB && currentVfo == vfoB))
-    {
-        func = ((rigCaps->commands.contains(funcSelectedFreq)) ? funcSelectedFreq: func);
-    }
-    else if ((vfo == vfoA && currentVfo == vfoB) || (vfo == vfoB && currentVfo == vfoA))
-    {
-        func = ((rigCaps->commands.contains(funcUnselectedFreq)) ? funcUnselectedFreq: func);
-        if (currentVfo == vfoSub && rigCaps->numReceiver>1)
-        {
-            currentRx=1;
-        }
-    }
-    else if (vfo == vfoMain || vfo == vfoSub)
-    {
-        func = ((rigCaps->commands.contains(funcFreq)) ? funcFreq: func);
-    }
-     //qDebug(logRigCtlD) << "Frequency function is:" << funcString[func];
-    return func;
-}
-
-funcs rigCtlClient::getModeFunc(vfo_t vfo, bool set)
-{
-    funcs func = ((rigCaps->commands.contains(funcMode)) ? funcMode: funcModeGet) ;
-
-    currentRx=0;
-
-    if (set)
-        func = ((rigCaps->commands.contains(funcMode)) ? funcMode: funcModeSet) ;
-
-    if ((vfo == vfoA && currentVfo == vfoA) || (vfo == vfoB && currentVfo == vfoB))
-    {
-        func = ((rigCaps->commands.contains(funcSelectedMode)) ? funcSelectedMode: func);
-    }
-    else if ((vfo == vfoA && currentVfo == vfoB) || (vfo == vfoB && currentVfo == vfoA))
-    {
-        func = ((rigCaps->commands.contains(funcUnselectedMode)) ? funcUnselectedMode: func);
-    }
-    else if (vfo == vfoMain || vfo == vfoSub)
-    {
-        func = ((rigCaps->commands.contains(funcMode)) ? funcMode: func);
-        if (currentVfo == vfoSub && rigCaps->numReceiver>1)
-        {
-            currentRx=1;
-        }
-    }
-    //qDebug(logRigCtlD) << "Mode function is:" << funcString[func];
-    return func;
-}
-
 QStringList rigCtlClient::buildPrefixes(commandStruct cmd,bool extended)
 {
     QStringList prefixes;
@@ -1641,6 +1536,7 @@ int rigCtlClient::power2mW(QStringList& response, bool extended, const commandSt
 {
     int ret=RIG_OK;
     QStringList prefixes = buildPrefixes(cmd,extended);
+    rigStateType state = queue->getState();
     if (params.size() >= 2)
     {
         bool ok=false;
@@ -1651,7 +1547,7 @@ int rigCtlClient::power2mW(QStringList& response, bool extended, const commandSt
             if (freq == 0)
             {
                 // Frequency was zero, so get cached freq
-                freq = queue->getCache(getFreqFunc(currentVfo,false),currentRx).value.value<freqt>().Hz;
+                freq = queue->getCache(queue->getVfoCommand(state.vfo,state.receiver,false).freqFunc,state.receiver).value.value<freqt>().Hz;
             }
             for (auto &b: rigCaps->bands)
             {
@@ -1680,6 +1576,7 @@ int rigCtlClient::mW2power(QStringList& response, bool extended, const commandSt
 {
     int ret=RIG_OK;
     QStringList prefixes = buildPrefixes(cmd,extended);
+    rigStateType state = queue->getState();
     if (params.size() > 2)
     {
         bool ok=false;
@@ -1690,7 +1587,7 @@ int rigCtlClient::mW2power(QStringList& response, bool extended, const commandSt
             if (freq == 0)
             {
                 // Frequency was zero, so get cached freq
-                freq = queue->getCache(getFreqFunc(currentVfo,false),currentRx).value.value<freqt>().Hz;
+                freq = queue->getCache(queue->getVfoCommand(state.vfo,state.receiver,false).freqFunc,state.receiver).value.value<freqt>().Hz;
             }
             for (auto &b: rigCaps->bands)
             {
@@ -1716,50 +1613,3 @@ int rigCtlClient::mW2power(QStringList& response, bool extended, const commandSt
 
     return ret;
 }
-
-float rigCtlClient::rawToFloat(int raw, const cal_table_float_t *cal)
-{
-
-    int i;
-    float val;
-
-    if (cal->size == 0)
-    {
-        return raw;
-    }
-
-    for (i = 0; i<cal->size; i++)
-    {
-        if (raw < cal->table[i].raw)
-        {
-            break;
-        }
-    }
-
-    if (raw == cal->table[i - 1].raw)
-    {
-        return cal->table[i - 1].val;
-    }
-
-    if (i == 0)
-    {
-        return cal->table[0].val;
-    }
-
-    if (i >= cal->size)
-    {
-        return cal->table[i - 1].val;
-    }
-
-    if (cal->table[i].raw == cal->table[i - 1].raw)
-    {
-        return cal->table[i].val;
-    }
-
-    val = ((cal->table[i].raw - raw)
-                     * (float)(cal->table[i].val - cal->table[i - 1].val))
-                    / (float)(cal->table[i].raw - cal->table[i - 1].raw);
-
-    return float(cal->table[i].val - val);
-}
-
